@@ -45,7 +45,7 @@
 from __future__ import absolute_import
 from __future__ import with_statement
 
-VERSION = (2, 7, 3, 3)
+VERSION = (2, 7, 3, 4)
 __version__ = ".".join(map(str, VERSION[0:4])) + "".join(VERSION[4:])
 __author__ = 'R Oudkerk / Python Software Foundation'
 __author_email__ = 'python-dev@python.org'
@@ -86,7 +86,7 @@ from .process import Process, current_process, active_children
 from .util import SUBDEBUG, SUBWARNING
 
 # This is down here because _billiard uses BufferTooShort
-from ._ext import supports_exec
+from ._ext import supports_exec, ensure_multiprocessing  # noqa
 
 #
 # Definitions not depending on native semaphores
