@@ -1,13 +1,40 @@
 #
 # A higher level module for using sockets (or Windows named pipes)
 #
-# billiard/connection.py
+# multiprocessing/connection.py
 #
-# Copyright (c) 2006-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
+from __future__ import with_statement
 
-__all__ = [ 'Client', 'Listener', 'Pipe' ]
+__all__ = ['Client', 'Listener', 'Pipe']
 
 import os
 import sys
@@ -17,12 +44,21 @@ import time
 import tempfile
 import itertools
 
-import _billiard
-from . import current_process, AuthenticationError
+from . import AuthenticationError
+from ._ext import _billiard, win32
 from .util import get_temp_dir, Finalize, sub_debug, debug
 from .forking import duplicate, close
 from .compat import bytes
 
+try:
+    WindowsError = WindowsError  # noqa
+except NameError:
+    WindowsError = None  # noqa
+
+
+# global set later
+xmlrpclib = None
+
 
 #
 #
@@ -49,6 +85,7 @@ if sys.platform == 'win32':
 def _init_timeout(timeout=CONNECTION_TIMEOUT):
     return time.time() + timeout
 
+
 def _check_timeout(t):
     return time.time() > t
 
@@ -56,6 +93,7 @@ def _check_timeout(t):
 #
 #
 
+
 def arbitrary_address(family):
     '''
     Return an arbitrary free address for the given family
@@ -71,6 +109,19 @@ def arbitrary_address(family):
         raise ValueError('unrecognized family')
 
 
+def _validate_family(family):
+    '''
+    Checks if the family is valid for the current environment.
+    '''
+    if sys.platform != 'win32' and family == 'AF_PIPE':
+        raise ValueError('Family %s is not recognized' % family)
+
+    if sys.platform == 'win32' and family == 'AF_UNIX':
+        # double check
+        if not hasattr(socket, family):
+            raise ValueError('Family %s is not recognized' % family)
+
+
 def address_type(address):
     '''
     Return the types of the address
@@ -90,6 +141,7 @@ def address_type(address):
 # Public functions
 #
 
+
 class Listener(object):
     '''
     Returns a listener object.
@@ -102,13 +154,14 @@ class Listener(object):
                  or default_family
         address = address or arbitrary_address(family)
 
+        _validate_family(family)
         if family == 'AF_PIPE':
             self._listener = PipeListener(address, backlog)
         else:
             self._listener = SocketListener(address, family, backlog)
 
         if authkey is not None and not isinstance(authkey, bytes):
-            raise TypeError, 'authkey should be a byte string'
+            raise TypeError('authkey should be a byte string')
 
         self._authkey = authkey
 
@@ -139,13 +192,14 @@ def Client(address, family=None, authkey=None):
     Returns a connection to the address of a `Listener`
     '''
     family = family or address_type(address)
+    _validate_family(family)
     if family == 'AF_PIPE':
         c = PipeClient(address)
     else:
         c = SocketClient(address)
 
     if authkey is not None and not isinstance(authkey, bytes):
-        raise TypeError, 'authkey should be a byte string'
+        raise TypeError('authkey should be a byte string')
 
     if authkey is not None:
         answer_challenge(c, authkey)
@@ -175,9 +229,7 @@ if sys.platform != 'win32':
 
 else:
 
-    from _billiard import win32
-
-    def Pipe(duplex=True):
+    def Pipe(duplex=True):  # noqa
         '''
         Returns pair of connection objects at either end of a pipe
         '''
@@ -219,16 +271,24 @@ else:
 # Definitions for connections based on sockets
 #
 
+
 class SocketListener(object):
     '''
     Representation of a socket which is bound to an address and listening
     '''
     def __init__(self, address, family, backlog=1):
         self._socket = socket.socket(getattr(socket, family))
-        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self._socket.bind(address)
-        self._socket.listen(backlog)
-        self._address = self._socket.getsockname()
+        try:
+            # SO_REUSEADDR has different semantics on Windows (Issue #2550).
+            if os.name == 'posix':
+                self._socket.setsockopt(socket.SOL_SOCKET,
+                                        socket.SO_REUSEADDR, 1)
+            self._socket.bind(address)
+            self._socket.listen(backlog)
+            self._address = self._socket.getsockname()
+        except OSError:
+            self._socket.close()
+            raise
         self._family = family
         self._last_accepted = None
 
@@ -257,26 +317,26 @@ def SocketClient(address):
     Return a connection object connected to the socket given by `address`
     '''
     family = address_type(address)
-    s = socket.socket( getattr(socket, family) )
-    t = _init_timeout()
+    s = socket.socket(getattr(socket, family))
+    try:
+        t = _init_timeout()
 
-    while 1:
-        try:
-            s.connect(address)
-        except socket.error, e:
-            if e.args[0] != errno.ECONNREFUSED or _check_timeout(t):
-                debug('failed to connect to address %s', address)
-                raise
-            time.sleep(0.01)
+        while 1:
+            try:
+                s.connect(address)
+            except socket.error, e:
+                if e.args[0] != errno.ECONNREFUSED or _check_timeout(t):
+                    debug('failed to connect to address %s', address)
+                    raise
+                time.sleep(0.01)
+            else:
+                break
         else:
-            break
-    else:
-        raise
-
-    fd = duplicate(s.fileno())
-    conn = _billiard.Connection(fd)
-    s.close()
-    return conn
+            raise
+        fd = duplicate(s.fileno())
+        return _billiard.Connection(fd)
+    finally:
+        s.close()
 
 #
 # Definitions for connections based on named pipes
@@ -340,11 +400,12 @@ if sys.platform == 'win32':
                 win32.WaitNamedPipe(address, 1000)
                 h = win32.CreateFile(
                     address, win32.GENERIC_READ | win32.GENERIC_WRITE,
-                    0, win32.NULL, win32.OPEN_EXISTING, 0, win32.NULL
+                    0, win32.NULL, win32.OPEN_EXISTING, 0, win32.NULL,
                     )
             except WindowsError, e:
-                if e.args[0] not in (win32.ERROR_SEM_TIMEOUT,
-                                     win32.ERROR_PIPE_BUSY) or _check_timeout(t):
+                if e.args[0] not in (
+                        win32.ERROR_SEM_TIMEOUT,
+                        win32.ERROR_PIPE_BUSY) or _check_timeout(t):
                     raise
             else:
                 break
@@ -366,6 +427,7 @@ CHALLENGE = bytes('#CHALLENGE#')
 WELCOME = bytes('#WELCOME#')
 FAILURE = bytes('#FAILURE#')
 
+
 def deliver_challenge(connection, authkey):
     import hmac
     assert isinstance(authkey, bytes)
@@ -379,6 +441,7 @@ def deliver_challenge(connection, authkey):
         connection.send_bytes(FAILURE)
         raise AuthenticationError('digest received was wrong')
 
+
 def answer_challenge(connection, authkey):
     import hmac
     assert isinstance(authkey, bytes)
@@ -395,6 +458,7 @@ def answer_challenge(connection, authkey):
 # Support for using xmlrpclib for serialization
 #
 
+
 class ConnectionWrapper(object):
     def __init__(self, conn, dumps, loads):
         self._conn = conn
@@ -403,28 +467,34 @@ class ConnectionWrapper(object):
         for attr in ('fileno', 'close', 'poll', 'recv_bytes', 'send_bytes'):
             obj = getattr(conn, attr)
             setattr(self, attr, obj)
+
     def send(self, obj):
         s = self._dumps(obj)
         self._conn.send_bytes(s)
+
     def recv(self):
         s = self._conn.recv_bytes()
         return self._loads(s)
 
+
 def _xml_dumps(obj):
-    return xmlrpclib.dumps((obj,), None, None, None, 1).encode('utf8')
+    return xmlrpclib.dumps((obj,), None, None, None, 1).encode('utf-8')
+
 
 def _xml_loads(s):
-    (obj,), method = xmlrpclib.loads(s.decode('utf8'))
+    (obj,), method = xmlrpclib.loads(s.decode('utf-8'))
     return obj
 
+
 class XmlListener(Listener):
     def accept(self):
         global xmlrpclib
-        import xmlrpclib
+        xmlrpclib = __import__("xmlrpclib")
         obj = Listener.accept(self)
         return ConnectionWrapper(obj, _xml_dumps, _xml_loads)
 
+
 def XmlClient(*args, **kwds):
     global xmlrpclib
-    import xmlrpclib
+    xmlrpclib = __import__("xmlrpclib")
     return ConnectionWrapper(Client(*args, **kwds), _xml_dumps, _xml_loads)
