@@ -2,50 +2,87 @@
 # Module providing the `SyncManager` class for dealing
 # with shared objects
 #
-# billiard/managers.py
+# multiprocessing/managers.py
 #
-# Copyright (c) 2006-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
+from __future__ import with_statement
 
-__all__ = [ 'BaseManager', 'SyncManager', 'BaseProxy', 'Token' ]
+__all__ = ['BaseManager', 'SyncManager', 'BaseProxy', 'Token']
 
 #
 # Imports
 #
 
-import os
 import sys
-import weakref
 import threading
 import array
 import Queue
 
 from traceback import format_exc
+from time import time as _time
+
 from . import Process, current_process, active_children, Pool, util, connection
 from .process import AuthenticationString
-from .forking import exit, Popen, assert_spawning, ForkingPickler
-from .util import Finalize, info
-
-try:
-    from cPickle import PicklingError
-except ImportError:
-    from pickle import PicklingError
+from .forking import exit, Popen, ForkingPickler
+from .util import Finalize, error, info
 
 #
 # Register some things for pickling
 #
 
+
 def reduce_array(a):
     return array.array, (a.typecode, a.tostring())
 ForkingPickler.register(array.array, reduce_array)
 
-view_types = [type(getattr({}, name)()) for name in ('items','keys','values')]
+view_types = [type(getattr({}, name)())
+                 for name in ('items', 'keys', 'values')]
+if view_types[0] is not list:  # only needed in Py3.0
+
+    def rebuild_as_list(obj):
+        return list, (list(obj), )
+    for view_type in view_types:
+        ForkingPickler.register(view_type, rebuild_as_list)
+        try:
+            import copyreg
+        except ImportError:
+            pass
+        else:
+            copyreg.pickle(view_type, rebuild_as_list)
 
 #
 # Type for identifying shared objects
 #
 
+
 class Token(object):
     '''
     Type to uniquely indentify a shared object
@@ -69,6 +106,7 @@ class Token(object):
 # Function for communication with a manager's server process
 #
 
+
 def dispatch(c, id, methodname, args=(), kwds={}):
     '''
     Send a message to manager using connection `c` and return response
@@ -79,6 +117,7 @@ def dispatch(c, id, methodname, args=(), kwds={}):
         return result
     raise convert_to_error(kind, result)
 
+
 def convert_to_error(kind, result):
     if kind == '#ERROR':
         return result
@@ -91,14 +130,17 @@ def convert_to_error(kind, result):
     else:
         return ValueError('Unrecognized message type')
 
+
 class RemoteError(Exception):
+
     def __str__(self):
-        return ('\n' + '-'*75 + '\n' + str(self.args[0]) + '-'*75)
+        return ('\n' + '-' * 75 + '\n' + str(self.args[0]) + '-' * 75)
 
 #
 # Functions for finding the method names of an object
 #
 
+
 def all_methods(obj):
     '''
     Return a list of names of methods of `obj`
@@ -106,10 +148,11 @@ def all_methods(obj):
     temp = []
     for name in dir(obj):
         func = getattr(obj, name)
-        if hasattr(func, '__call__'):
+        if callable(func):
             temp.append(name)
     return temp
 
+
 def public_methods(obj):
     '''
     Return a list of names of methods of `obj` which do not start with '_'
@@ -120,6 +163,7 @@ def public_methods(obj):
 # Server which is run in a process controlled by a manager
 #
 
+
 class Server(object):
     '''
     Server class which runs in a process controlled by a manager object
@@ -134,7 +178,7 @@ class Server(object):
         Listener, Client = listener_client[serializer]
 
         # do authentication later
-        self.listener = Listener(address=address, backlog=5)
+        self.listener = Listener(address=address, backlog=16)
         self.address = self.listener.address
 
         self.id_to_obj = {'0': (None, ())}
@@ -191,9 +235,9 @@ class Server(object):
                 c.send(('#TRACEBACK', format_exc()))
             except Exception:
                 pass
-            util.info('Failure to send message: %r', msg)
-            util.info(' ... request was %r', request)
-            util.info(' ... exception was %r', e)
+            info('Failure to send message: %r', msg)
+            info(' ... request was %r', request)
+            info(' ... exception was %r', e)
 
         c.close()
 
@@ -264,10 +308,10 @@ class Server(object):
                 except Exception, e:
                     send(('#UNSERIALIZABLE', repr(msg)))
             except Exception, e:
-                util.info('exception in thread serving %r',
+                info('exception in thread serving %r',
                         threading.current_thread().name)
-                util.info(' ... message was %r', msg)
-                util.info(' ... exception was %r', e)
+                info(' ... message was %r', msg)
+                info(' ... exception was %r', e)
                 conn.close()
                 sys.exit(1)
 
@@ -281,10 +325,10 @@ class Server(object):
         return repr(obj)
 
     fallback_mapping = {
-        '__str__':fallback_str,
-        '__repr__':fallback_repr,
-        '#GETVALUE':fallback_getvalue
-        }
+        '__str__': fallback_str,
+        '__repr__': fallback_repr,
+        '#GETVALUE': fallback_getvalue,
+    }
 
     def dummy(self, c):
         pass
@@ -293,8 +337,7 @@ class Server(object):
         '''
         Return some info --- useful to spot problems with refcounting
         '''
-        self.mutex.acquire()
-        try:
+        with self.mutex:
             result = []
             keys = self.id_to_obj.keys()
             keys.sort()
@@ -304,8 +347,6 @@ class Server(object):
                                   (ident, self.id_to_refcount[ident],
                                    str(self.id_to_obj[ident][0])[:75]))
             return '\n'.join(result)
-        finally:
-            self.mutex.release()
 
     def number_of_objects(self, c):
         '''
@@ -338,10 +379,11 @@ class Server(object):
                     p.join()
 
                 util._run_finalizers()
-                util.info('manager exiting with exitcode 0')
+                info('manager exiting with exitcode 0')
             except:
-                import traceback
-                traceback.print_exc()
+                if not error("Error while manager shutdown", exc_info=True):
+                    import traceback
+                    traceback.print_exc()
         finally:
             exit(0)
 
@@ -349,8 +391,7 @@ class Server(object):
         '''
         Create a new shared object and return its id
         '''
-        self.mutex.acquire()
-        try:
+        with self.mutex:
             callable, exposed, method_to_typeid, proxytype = \
                       self.registry[typeid]
 
@@ -380,8 +421,6 @@ class Server(object):
             # has been created.
             self.incref(c, ident)
             return ident, tuple(exposed)
-        finally:
-            self.mutex.release()
 
     def get_methods(self, c, token):
         '''
@@ -398,27 +437,22 @@ class Server(object):
         self.serve_client(c)
 
     def incref(self, c, ident):
-        self.mutex.acquire()
-        try:
+        with self.mutex:
             self.id_to_refcount[ident] += 1
-        finally:
-            self.mutex.release()
 
     def decref(self, c, ident):
-        self.mutex.acquire()
-        try:
+        with self.mutex:
             assert self.id_to_refcount[ident] >= 1
             self.id_to_refcount[ident] -= 1
             if self.id_to_refcount[ident] == 0:
                 del self.id_to_obj[ident], self.id_to_refcount[ident]
                 util.debug('disposing of obj with id %r', ident)
-        finally:
-            self.mutex.release()
 
 #
 # Class to represent state of a manager
 #
 
+
 class State(object):
     __slots__ = ['value']
     INITIAL = 0
@@ -430,14 +464,15 @@ class State(object):
 #
 
 listener_client = {
-    'pickle' : (connection.Listener, connection.Client),
-    'xmlrpclib' : (connection.XmlListener, connection.XmlClient)
-    }
+    'pickle': (connection.Listener, connection.Client),
+    'xmlrpclib': (connection.XmlListener, connection.XmlClient),
+}
 
 #
 # Definition of BaseManager
 #
 
+
 class BaseManager(object):
     '''
     Base class for managers
@@ -482,7 +517,7 @@ class BaseManager(object):
         '''
         assert self._state.value == State.INITIAL
 
-        if initializer is not None and not hasattr(initializer, '__call__'):
+        if initializer is not None and not callable(initializer):
             raise TypeError('initializer must be a callable')
 
         # pipe over which we will retrieve address of server
@@ -495,7 +530,7 @@ class BaseManager(object):
                   self._serializer, writer, initializer, initargs),
             )
         ident = ':'.join(str(i) for i in self._process._identity)
-        self._process.name = type(self).__name__  + '-' + ident
+        self._process.name = type(self).__name__ + '-' + ident
         self._process.start()
 
         # get address of server
@@ -505,7 +540,7 @@ class BaseManager(object):
 
         # register a finalizer
         self._state.value = State.STARTED
-        self.shutdown = util.Finalize(
+        self.shutdown = Finalize(
             self, type(self)._finalize_manager,
             args=(self._process, self._address, self._authkey,
                   self._state, self._Client),
@@ -529,7 +564,7 @@ class BaseManager(object):
         writer.close()
 
         # run the manager
-        util.info('manager serving at %r', server.address)
+        info('manager serving at %r', server.address)
         server.serve_forever()
 
     def _create(self, typeid, *args, **kwds):
@@ -539,7 +574,8 @@ class BaseManager(object):
         assert self._state.value == State.STARTED, 'server not yet started'
         conn = self._Client(self._address, authkey=self._authkey)
         try:
-            id, exposed = dispatch(conn, None, 'create', (typeid,)+args, kwds)
+            id, exposed = dispatch(conn, None, 'create',
+                                   (typeid,) + args, kwds)
         finally:
             conn.close()
         return Token(typeid, self._address, id), exposed
@@ -582,7 +618,7 @@ class BaseManager(object):
         Shutdown the manager process; will be registered as a finalizer
         '''
         if process.is_alive():
-            util.info('sending shutdown message to manager')
+            info('sending shutdown message to manager')
             try:
                 conn = _Client(address, authkey=authkey)
                 try:
@@ -594,13 +630,13 @@ class BaseManager(object):
 
             process.join(timeout=0.2)
             if process.is_alive():
-                util.info('manager still alive')
+                info('manager still alive')
                 if hasattr(process, 'terminate'):
-                    util.info('trying to `terminate()` manager process')
+                    info('trying to `terminate()` manager process')
                     process.terminate()
                     process.join(timeout=0.1)
                     if process.is_alive():
-                        util.info('manager still alive after terminate')
+                        info('manager still alive after terminate')
 
         state.value = State.SHUTDOWN
         try:
@@ -654,9 +690,12 @@ class BaseManager(object):
 # Subclass of set which get cleared after a fork
 #
 
+
 class ProcessLocalSet(set):
+
     def __init__(self):
         util.register_after_fork(self, lambda obj: obj.clear())
+
     def __reduce__(self):
         return type(self), ()
 
@@ -664,6 +703,7 @@ class ProcessLocalSet(set):
 # Definition of BaseProxy
 #
 
+
 class BaseProxy(object):
     '''
     A base for proxies of shared objects
@@ -762,7 +802,7 @@ class BaseProxy(object):
 
         state = self._manager and self._manager._state
 
-        self._close = util.Finalize(
+        self._close = Finalize(
             self, BaseProxy._decref,
             args=(self._token, self._authkey, state,
                   self._tls, self._idset, self._Client),
@@ -800,7 +840,7 @@ class BaseProxy(object):
             self._incref()
         except Exception, e:
             # the proxy may just be for a manager which has shutdown
-            util.info('incref failed: %s' % e)
+            info('incref failed: %s', e)
 
     def __reduce__(self):
         kwds = {}
@@ -835,6 +875,7 @@ class BaseProxy(object):
 # Function used for unpickling
 #
 
+
 def RebuildProxy(func, token, serializer, kwds):
     '''
     Function used for unpickling proxy objects.
@@ -856,6 +897,7 @@ def RebuildProxy(func, token, serializer, kwds):
 # Functions to create proxies and proxy types
 #
 
+
 def MakeProxyType(name, exposed, _cache={}):
     '''
     Return an proxy type whose methods are given by `exposed`
@@ -869,8 +911,8 @@ def MakeProxyType(name, exposed, _cache={}):
     dic = {}
 
     for meth in exposed:
-        exec '''def %s(self, *args, **kwds):
-        return self._callmethod(%r, args, kwds)''' % (meth, meth) in dic
+        exec('''def %s(self, *args, **kwds):
+        return self._callmethod(%r, args, kwds)''' % (meth, meth), dic)
 
     ProxyType = type(name, (BaseProxy,), dic)
     ProxyType._exposed_ = exposed
@@ -907,9 +949,12 @@ def AutoProxy(token, serializer, manager=None, authkey=None,
 # Types/callables which we will register with SyncManager
 #
 
+
 class Namespace(object):
+
     def __init__(self, **kwds):
         self.__dict__.update(kwds)
+
     def __repr__(self):
         items = self.__dict__.items()
         temp = []
@@ -919,18 +964,25 @@ class Namespace(object):
         temp.sort()
         return 'Namespace(%s)' % str.join(', ', temp)
 
+
 class Value(object):
+
     def __init__(self, typecode, value, lock=True):
         self._typecode = typecode
         self._value = value
+
     def get(self):
         return self._value
+
     def set(self, value):
         self._value = value
+
     def __repr__(self):
-        return '%s(%r, %r)'%(type(self).__name__, self._typecode, self._value)
+        return '%s(%r, %r)' % (type(self).__name__,
+                               self._typecode, self._value)
     value = property(get, set)
 
+
 def Array(typecode, sequence, lock=True):
     return array.array(typecode, sequence)
 
@@ -938,68 +990,110 @@ def Array(typecode, sequence, lock=True):
 # Proxy types used by SyncManager
 #
 
+
 class IteratorProxy(BaseProxy):
-    # XXX remove methods for Py3.0 and Py2.6
-    _exposed_ = ('__next__', 'next', 'send', 'throw', 'close')
+    if sys.version_info[0] == 3:
+        _exposed = ('__next__', 'send', 'throw', 'close')
+    else:
+        _exposed_ = ('__next__', 'next', 'send', 'throw', 'close')
+
+        def next(self, *args):
+            return self._callmethod('next', args)
+
     def __iter__(self):
         return self
+
     def __next__(self, *args):
         return self._callmethod('__next__', args)
-    def next(self, *args):
-        return self._callmethod('next', args)
+
     def send(self, *args):
         return self._callmethod('send', args)
+
     def throw(self, *args):
         return self._callmethod('throw', args)
+
     def close(self, *args):
         return self._callmethod('close', args)
 
 
 class AcquirerProxy(BaseProxy):
     _exposed_ = ('acquire', 'release')
+
     def acquire(self, blocking=True):
         return self._callmethod('acquire', (blocking,))
+
     def release(self):
         return self._callmethod('release')
+
     def __enter__(self):
         return self._callmethod('acquire')
+
     def __exit__(self, exc_type, exc_val, exc_tb):
         return self._callmethod('release')
 
 
 class ConditionProxy(AcquirerProxy):
-    # XXX will Condition.notfyAll() name be available in Py3.0?
     _exposed_ = ('acquire', 'release', 'wait', 'notify', 'notify_all')
+
     def wait(self, timeout=None):
         return self._callmethod('wait', (timeout,))
+
     def notify(self):
         return self._callmethod('notify')
+
     def notify_all(self):
         return self._callmethod('notify_all')
 
+    def wait_for(self, predicate, timeout=None):
+        result = predicate()
+        if result:
+            return result
+        if timeout is not None:
+            endtime = _time() + timeout
+        else:
+            endtime = None
+            waittime = None
+        while not result:
+            if endtime is not None:
+                waittime = endtime - _time()
+                if waittime <= 0:
+                    break
+            self.wait(waittime)
+            result = predicate()
+        return result
+
+
 class EventProxy(BaseProxy):
     _exposed_ = ('is_set', 'set', 'clear', 'wait')
+
     def is_set(self):
         return self._callmethod('is_set')
+
     def set(self):
         return self._callmethod('set')
+
     def clear(self):
         return self._callmethod('clear')
+
     def wait(self, timeout=None):
         return self._callmethod('wait', (timeout,))
 
+
 class NamespaceProxy(BaseProxy):
     _exposed_ = ('__getattribute__', '__setattr__', '__delattr__')
+
     def __getattr__(self, key):
         if key[0] == '_':
             return object.__getattribute__(self, key)
         callmethod = object.__getattribute__(self, '_callmethod')
         return callmethod('__getattribute__', (key,))
+
     def __setattr__(self, key, value):
         if key[0] == '_':
             return object.__setattr__(self, key, value)
         callmethod = object.__getattribute__(self, '_callmethod')
         return callmethod('__setattr__', (key, value))
+
     def __delattr__(self, key):
         if key[0] == '_':
             return object.__delattr__(self, key)
@@ -1009,8 +1103,10 @@ class NamespaceProxy(BaseProxy):
 
 class ValueProxy(BaseProxy):
     _exposed_ = ('get', 'set')
+
     def get(self):
         return self._callmethod('get')
+
     def set(self, value):
         return self._callmethod('set', (value,))
     value = property(get, set)
@@ -1023,10 +1119,14 @@ BaseListProxy = MakeProxyType('BaseListProxy', (
     'append', 'count', 'extend', 'index', 'insert', 'pop', 'remove',
     'reverse', 'sort', '__imul__'
     ))                  # XXX __getslice__ and __setslice__ unneeded in Py3.0
+
+
 class ListProxy(BaseListProxy):
+
     def __iadd__(self, value):
         self._callmethod('extend', (value,))
         return self
+
     def __imul__(self, value):
         self._callmethod('__imul__', (value,))
         return self
@@ -1046,11 +1146,12 @@ ArrayProxy = MakeProxyType('ArrayProxy', (
 
 PoolProxy = MakeProxyType('PoolProxy', (
     'apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join',
-    'map', 'map_async', 'terminate'
+    'map', 'map_async', 'starmap', 'starmap_async', 'terminate',
     ))
 PoolProxy._method_to_typeid_ = {
     'apply_async': 'AsyncResult',
     'map_async': 'AsyncResult',
+    'starmap_async': 'AsyncResult',
     'imap': 'Iterator',
     'imap_unordered': 'Iterator'
     }
@@ -1059,6 +1160,7 @@ PoolProxy._method_to_typeid_ = {
 # Definition of SyncManager
 #
 
+
 class SyncManager(BaseManager):
     '''
     Subclass of `BaseManager` which supports a number of shared object types.
