@@ -4,9 +4,36 @@
 #
 # multiprocessing/pool.py
 #
-# Copyright (c) 2007-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
+from __future__ import with_statement
 
 #
 # Imports
@@ -68,6 +95,10 @@ def mapstar(args):
     return map(*args)
 
 
+def starmapstar(args):
+    return list(itertools.starmap(args[0], args[1]))
+
+
 def error(msg, *args, **kwargs):
     if util._logger:
         util._logger.error(msg, *args, **kwargs)
@@ -90,7 +121,7 @@ class LaxBoundedSemaphore(threading._Semaphore):
         _Semaphore.__init__(self, value, verbose)
         self._initial_value = value
 
-    if sys.version_info >= (3, 0):
+    if sys.version_info[0] == 3:
 
         def release(self):
             if self._value < self._initial_value:
@@ -121,18 +152,19 @@ class LaxBoundedSemaphore(threading._Semaphore):
 
 
 class MaybeEncodingError(Exception):
-    """Wraps unpickleable object."""
+    """Wraps possible unpickleable errors, so they can be
+    safely sent through the socket."""
 
     def __init__(self, exc, value):
-        self.exc = str(exc)
+        self.exc = repr(exc)
         self.value = repr(value)
-        Exception.__init__(self, self.exc, self.value)
+        super(MaybeEncodingError, self).__init__(self.exc, self.value)
 
     def __repr__(self):
         return "<MaybeEncodingError: %s>" % str(self)
 
     def __str__(self):
-        return "Error sending result: '%s'. Reason: '%s'." % (
+        return "Error sending result: '%r'. Reason: '%r'." % (
                     self.value, self.exc)
 
 
@@ -223,7 +255,7 @@ def worker(inqueue, outqueue, initializer=None, initargs=(),
             put((READY, (job, i, (False, einfo))))
 
         completed += 1
-    debug('worker exiting after %d tasks' % completed)
+    debug('worker exiting after %d tasks', completed)
 
 #
 # Class representing a process pool
@@ -241,7 +273,7 @@ class PoolThread(threading.Thread):
         try:
             return self.body()
         except Exception, exc:
-            error("Thread %r crashed: %r" % (self.__class__.__name__, exc, ),
+            error("Thread %r crashed: %r", type(self).__name__, exc,
                   exc_info=True)
             os._exit(1)
 
@@ -260,6 +292,9 @@ class Supervisor(PoolThread):
 
     def body(self):
         debug('worker handler starting')
+
+        # Keep maintaing workers until the cache gets drained, unless the pool
+        # is termianted
         while self._state == RUN and self.pool._state == RUN:
             self.pool._maintain_pool()
             time.sleep(0.8)
@@ -344,7 +379,7 @@ class TimeoutHandler(PoolThread):
                 return True
 
         def _on_soft_timeout(job, i, soft_timeout):
-            debug('soft time limit exceeded for %i' % i)
+            debug('soft time limit exceeded for %i', i)
             process, _index = _process_by_pid(job._worker_pid)
             if not process:
                 return
@@ -456,14 +491,14 @@ class ResultHandler(PoolThread):
             try:
                 state_handlers[state](*args)
             except KeyError:
-                debug("Unknown job state: %s (args=%s)" % (state, args))
+                debug("Unknown job state: %s (args=%s)", state, args)
 
         debug('result handler starting')
         while 1:
             try:
                 ready, task = poll(1.0)
             except (IOError, EOFError), exc:
-                debug('result handler got %r -- exiting' % (exc, ))
+                debug('result handler got %r -- exiting', exc)
                 return
 
             if self._state:
@@ -483,7 +518,7 @@ class ResultHandler(PoolThread):
             try:
                 ready, task = poll(1.0)
             except (IOError, EOFError), exc:
-                debug('result handler got %r -- exiting' % (exc, ))
+                debug('result handler got %r -- exiting', exc)
                 return
 
             if ready:
@@ -503,8 +538,8 @@ class ResultHandler(PoolThread):
                         debug('result handler exiting: timed out')
                         break
                     debug('result handler: all workers terminated, '
-                          'timeout in %ss' % (
-                              abs(min(now - time_terminate - 5.0, 0))))
+                          'timeout in %ss',
+                              abs(min(now - time_terminate - 5.0, 0)))
 
         if hasattr(outqueue, '_reader'):
             debug('ensuring that outqueue is not full')
@@ -525,7 +560,7 @@ class ResultHandler(PoolThread):
 
 class Pool(object):
     '''
-    Class which supports an async version of the `apply()` builtin
+    Class which supports an async version of applying functions to arguments.
     '''
     Process = Process
     Supervisor = Supervisor
@@ -560,7 +595,7 @@ class Pool(object):
                 processes = 1
         self._processes = processes
 
-        if initializer is not None and not hasattr(initializer, '__call__'):
+        if initializer is not None and not callable(initializer):
             raise TypeError('initializer must be a callable')
 
         self._pool = []
@@ -602,7 +637,7 @@ class Pool(object):
             )
 
     def _create_worker_process(self):
-        sentinel = None
+        sentinel = Event()
         w = self.Process(
             target=worker,
             args=(self._inqueue, self._outqueue,
@@ -646,9 +681,9 @@ class Pool(object):
             worker = self._pool[i]
             if worker.exitcode is not None:
                 # worker exited
-                debug('Supervisor: cleaning up worker %d' % i)
+                debug('Supervisor: cleaning up worker %d', i)
                 worker.join()
-                debug('Supervisor: worked %d joined' % i)
+                debug('Supervisor: worked %d joined', i)
                 cleaned.append(worker.pid)
                 del self._pool[i]
                 del self._poolctrl[worker.pid]
@@ -681,13 +716,10 @@ class Pool(object):
             self._processes += 1
             if self._putlock:
                 cond = self._putlock._Semaphore__cond
-                cond.acquire()
-                try:
+                with cond:
                     self._putlock._initial_value += 1
                     self._putlock._Semaphore__value += 1
                     cond.notify()
-                finally:
-                    cond.release()
 
     def _iterinactive(self):
         for worker in self._pool:
@@ -733,34 +765,49 @@ class Pool(object):
     def _start_timeout_handler(self):
         # ensure more than one thread does not start the timeout handler
         # thread at once.
-        self._timeout_handler_mutex.acquire()
-        try:
+        with self._timeout_handler_mutex:
             if self._timeout_handler is None:
                 self._timeout_handler = self.TimeoutHandler(
                         self._pool, self._cache,
                         self.soft_timeout, self.timeout)
                 self._timeout_handler.start()
-        finally:
-            self._timeout_handler_mutex.release()
 
     def apply(self, func, args=(), kwds={}):
         '''
-        Equivalent of `apply()` builtin
+        Equivalent of `func(*args, **kwargs)`.
         '''
         assert self._state == RUN
         return self.apply_async(func, args, kwds).get()
 
+    def starmap(self, func, iterable, chunksize=None):
+        '''
+        Like `map()` method but the elements of the `iterable` are expected to
+        be iterables as well and will be unpacked as arguments. Hence
+        `func` and (a, b) becomes func(a, b).
+        '''
+        assert self._state == RUN
+        return self._map_async(func, iterable, starmapstar, chunksize).get()
+
+    def starmap_async(self, func, iterable, chunksize=None, callback=None,
+            error_callback=None):
+        '''
+        Asynchronous version of `starmap()` method.
+        '''
+        assert self._state == RUN
+        return self._map_async(func, iterable, starmapstar, chunksize,
+                               callback, error_callback)
+
     def map(self, func, iterable, chunksize=None):
         '''
-        Equivalent of `map()` builtin
+        Apply `func` to each element in `iterable`, collecting the results
+        in a list that is returned.
         '''
         assert self._state == RUN
         return self.map_async(func, iterable, chunksize).get()
 
     def imap(self, func, iterable, chunksize=1, lost_worker_timeout=None):
         '''
-        Equivalent of `itertools.imap()` -- can be MUCH slower
-        than `Pool.map()`
+        Equivalent of `map()` -- can be MUCH slower than `Pool.map()`.
         '''
         assert self._state == RUN
         lost_worker_timeout = lost_worker_timeout or self.lost_worker_timeout
@@ -782,7 +829,7 @@ class Pool(object):
     def imap_unordered(self, func, iterable, chunksize=1,
                        lost_worker_timeout=None):
         '''
-        Like `imap()` method but ordering of results is arbitrary
+        Like `imap()` method but ordering of results is arbitrary.
         '''
         assert self._state == RUN
         lost_worker_timeout = lost_worker_timeout or self.lost_worker_timeout
@@ -802,11 +849,11 @@ class Pool(object):
             return (item for chunk in result for item in chunk)
 
     def apply_async(self, func, args=(), kwds={},
-            callback=None, accept_callback=None, timeout_callback=None,
-            waitforslot=False, error_callback=None,
+            callback=None, error_callback=None, accept_callback=None,
+            timeout_callback=None, waitforslot=False,
             soft_timeout=None, timeout=None, lost_worker_timeout=None):
         '''
-        Asynchronous equivalent of `apply()` builtin.
+        Asynchronous equivalent of `apply()` method.
 
         Callback is called when the functions return value is ready.
         The accept callback is called when the job is accepted to be executed.
@@ -840,9 +887,18 @@ class Pool(object):
                                    func, args, kwds)], None))
             return result
 
-    def map_async(self, func, iterable, chunksize=None, callback=None):
+    def map_async(self, func, iterable, chunksize=None, callback=None,
+            error_callback=None):
         '''
-        Asynchronous equivalent of `map()` builtin
+        Asynchronous equivalent of `map()` method.
+        '''
+        return self._map_async(func, iterable, mapstar, chunksize,
+                callback, error_callback)
+
+    def _map_async(self, func, iterable, mapper, chunksize=None,
+            callback=None, error_callback=None):
+        '''
+        Helper function to implement map, starmap and their async counterparts.
         '''
         assert self._state == RUN
         if not hasattr(iterable, '__len__'):
@@ -856,8 +912,9 @@ class Pool(object):
             chunksize = 0
 
         task_batches = Pool._get_tasks(func, iterable, chunksize)
-        result = MapResult(self._cache, chunksize, len(iterable), callback)
-        self._taskqueue.put((((result._job, i, mapstar, (x,), {})
+        result = MapResult(self._cache, chunksize, len(iterable), callback,
+                           error_callback=error_callback)
+        self._taskqueue.put((((result._job, i, mapper, (x,), {})
                               for i, x in enumerate(task_batches)), None))
         return result
 
@@ -880,8 +937,8 @@ class Pool(object):
         if self._state == RUN:
             self._state = CLOSE
             self._worker_handler.close()
-            self._worker_handler.join()
             self._taskqueue.put(None)
+            self._worker_handler.join()
             if self._putlock:
                 self._putlock.clear()
 
@@ -901,7 +958,7 @@ class Pool(object):
         self._result_handler.join()
         debug('result handler joined')
         for i, p in enumerate(self._pool):
-            debug('joining worker %s/%s (%r)' % (i, len(self._pool), p, ))
+            debug('joining worker %s/%s (%r)', i, len(self._pool), p)
             p.join()
 
     def restart(self):
@@ -947,10 +1004,10 @@ class Pool(object):
                     p.terminate()
 
         debug('joining task handler')
-        task_handler.join(1e100)
+        task_handler.join()
 
         debug('joining result handler')
-        result_handler.join(1e100)
+        result_handler.join()
 
         if timeout_handler is not None:
             debug('joining timeout handler')
@@ -961,7 +1018,7 @@ class Pool(object):
             for p in pool:
                 if p.is_alive():
                     # worker has not yet exited
-                    debug('cleaning up worker %d' % p.pid)
+                    debug('cleaning up worker %d', p.pid)
                     p.join()
             debug('pool workers joined')
 DynamicPool = Pool
@@ -984,7 +1041,7 @@ class ApplyResult(object):
         self._ready = False
         self._callback = callback
         self._accept_callback = accept_callback
-        self._errback = error_callback
+        self._error_callback = error_callback
         self._timeout_callback = timeout_callback
         self._timeout = timeout
         self._soft_timeout = soft_timeout
@@ -1009,12 +1066,9 @@ class ApplyResult(object):
         return filter(None, [self._worker_pid])
 
     def wait(self, timeout=None):
-        self._cond.acquire()
-        try:
+        with self._cond:
             if not self._ready:
                 self._cond.wait(timeout)
-        finally:
-            self._cond.release()
 
     def get(self, timeout=None):
         self.wait(timeout)
@@ -1026,15 +1080,11 @@ class ApplyResult(object):
             raise self._value
 
     def _set(self, i, obj):
-        self._mutex.acquire()
-        try:
+        with self._mutex:
             self._success, self._value = obj
-            self._cond.acquire()
-            try:
+            with self._cond:
                 self._ready = True
                 self._cond.notify()
-            finally:
-                self._cond.release()
             if self._accepted:
                 self._cache.pop(self._job, None)
 
@@ -1042,15 +1092,12 @@ class ApplyResult(object):
             if self._callback and self._success:
                 safe_apply_callback(
                     self._callback, self._value)
-            if self._errback and not self._success:
+            if self._error_callback and not self._success:
                 safe_apply_callback(
-                    self._errback, self._value)
-        finally:
-            self._mutex.release()
+                    self._error_callback, self._value)
 
     def _ack(self, i, time_accepted, pid):
-        self._mutex.acquire()
-        try:
+        with self._mutex:
             self._accepted = True
             self._time_accepted = time_accepted
             self._worker_pid = pid
@@ -1059,8 +1106,6 @@ class ApplyResult(object):
             if self._accept_callback:
                 safe_apply_callback(
                     self._accept_callback, pid, time_accepted)
-        finally:
-            self._mutex.release()
 
 #
 # Class whose instances are returned by `Pool.map_async()`
@@ -1069,8 +1114,9 @@ class ApplyResult(object):
 
 class MapResult(ApplyResult):
 
-    def __init__(self, cache, chunksize, length, callback):
-        ApplyResult.__init__(self, cache, callback)
+    def __init__(self, cache, chunksize, length, callback, error_callback):
+        ApplyResult.__init__(self, cache, callback,
+                error_callback=error_callback)
         self._success = True
         self._length = length
         self._value = [None] * length
@@ -1094,24 +1140,19 @@ class MapResult(ApplyResult):
                     self._callback(self._value)
                 if self._accepted:
                     self._cache.pop(self._job, None)
-                self._cond.acquire()
-                try:
+                with self._cond:
                     self._ready = True
                     self._cond.notify()
-                finally:
-                    self._cond.release()
-
         else:
             self._success = False
             self._value = result
+            if self._error_callback:
+                self._error_callback(self._value)
             if self._accepted:
                 self._cache.pop(self._job, None)
-            self._cond.acquire()
-            try:
+            with self._cond:
                 self._ready = True
                 self._cond.notify()
-            finally:
-                self._cond.release()
 
     def _ack(self, i, time_accepted, pid):
         start = i * self._chunksize
@@ -1154,8 +1195,7 @@ class IMapIterator(object):
         return self
 
     def next(self, timeout=None):
-        self._cond.acquire()
-        try:
+        with self._cond:
             try:
                 item = self._items.popleft()
             except IndexError:
@@ -1170,8 +1210,6 @@ class IMapIterator(object):
                         self._ready = True
                         raise StopIteration
                     raise TimeoutError
-        finally:
-            self._cond.release()
 
         success, value = item
         if success:
@@ -1181,8 +1219,7 @@ class IMapIterator(object):
     __next__ = next                    # XXX
 
     def _set(self, i, obj):
-        self._cond.acquire()
-        try:
+        with self._cond:
             if self._index == i:
                 self._items.append(obj)
                 self._index += 1
@@ -1197,19 +1234,14 @@ class IMapIterator(object):
             if self._index == self._length:
                 self._ready = True
                 del self._cache[self._job]
-        finally:
-            self._cond.release()
 
     def _set_length(self, length):
-        self._cond.acquire()
-        try:
+        with self._cond:
             self._length = length
             if self._index == self._length:
                 self._ready = True
                 self._cond.notify()
                 del self._cache[self._job]
-        finally:
-            self._cond.release()
 
     def _ack(self, i, time_accepted, pid):
         self._worker_pids.append(pid)
@@ -1228,16 +1260,13 @@ class IMapIterator(object):
 class IMapUnorderedIterator(IMapIterator):
 
     def _set(self, i, obj):
-        self._cond.acquire()
-        try:
+        with self._cond:
             self._items.append(obj)
             self._index += 1
             self._cond.notify()
             if self._index == self._length:
                 self._ready = True
                 del self._cache[self._job]
-        finally:
-            self._cond.release()
 
 #
 #
@@ -1268,10 +1297,7 @@ class ThreadPool(Pool):
     @staticmethod
     def _help_stuff_finish(inqueue, task_handler, size):
         # put sentinels at head of inqueue to make workers finish
-        inqueue.not_empty.acquire()
-        try:
+        with inqueue.not_empty:
             inqueue.queue.clear()
             inqueue.queue.extend([None] * size)
             inqueue.not_empty.notify_all()
-        finally:
-            inqueue.not_empty.release()
