@@ -1,17 +1,42 @@
 #
 # Module implementing synchronization primitives
 #
-# billiard/synchronize.py
+# multiprocessing/synchronize.py
 #
-# Copyright (c) 2006-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
 
 __all__ = [
-    'Lock', 'RLock', 'Semaphore', 'BoundedSemaphore', 'Condition', 'Event'
-    ]
+    'Lock', 'RLock', 'Semaphore', 'BoundedSemaphore', 'Condition', 'Event',
+]
 
-import errno
 import itertools
 import os
 import signal
@@ -19,9 +44,9 @@ import sys
 import threading
 
 
-from time import time as _time, sleep as _sleep
+from time import time as _time
 
-import _billiard
+from ._ext import _billiard, ensure_SemLock
 from .process import current_process
 from .util import Finalize, register_after_fork, debug
 from .forking import assert_spawning, Popen
@@ -30,13 +55,7 @@ from .compat import bytes, closerange
 # Try to import the mp.synchronize module cleanly, if it fails
 # raise ImportError for platforms lacking a working sem_open implementation.
 # See issue 3770
-try:
-    from _billiard import SemLock
-except (ImportError):
-    raise ImportError("This platform lacks a functioning sem_open" +
-                      " implementation, therefore, the required" +
-                      " synchronization primitives needed will not" +
-                      " function, see issue 3770.")
+ensure_SemLock()
 
 #
 # Constants
@@ -45,37 +64,46 @@ except (ImportError):
 RECURSIVE_MUTEX, SEMAPHORE = range(2)
 SEM_VALUE_MAX = _billiard.SemLock.SEM_VALUE_MAX
 
-sem_unlink = _billiard.SemLock.sem_unlink
- 
+try:
+    sem_unlink = _billiard.SemLock.sem_unlink
+except AttributeError:
+    sem_unlink = None
+
 #
 # Base class for semaphores and mutexes; wraps `_billiard.SemLock`
 #
 
-class SemLock(object):
 
+class SemLock(object):
     _counter = itertools.count()
 
     def __init__(self, kind, value, maxvalue):
         from .forking import _forking_is_enabled
         unlink_immediately = _forking_is_enabled or sys.platform == 'win32'
-        sl = self._semlock = _billiard.SemLock(
-            kind, value, maxvalue, self._make_name(), unlink_immediately)
+        if sem_unlink:
+            sl = self._semlock = _billiard.SemLock(
+                kind, value, maxvalue, self._make_name(), unlink_immediately)
+        else:
+            sl = self._semlock = _billiard.SemLock(kind, value, maxvalue)
 
-        debug('created semlock with handle %s' % sl.handle)
+        debug('created semlock with handle %s', sl.handle)
         self._make_methods()
 
-        if sys.platform != 'win32':
-            def _after_fork(obj):
-                obj._semlock._after_fork()
-            register_after_fork(self, _after_fork)
+        if sem_unlink:
 
-        if self._semlock.name is not None:
-            # We only get here if we are on Unix with forking
-            # disabled.  When the object is garbage collected or the
-            # process shuts down we unlink the semaphore name
-            Finalize(self, sem_unlink, (self._semlock.name,), exitpriority=0)
-            # In case of abnormal termination unlink semaphore name
-            _cleanup_semaphore_if_leaked(self._semlock.name)
+            if sys.platform != 'win32':
+                def _after_fork(obj):
+                    obj._semlock._after_fork()
+                register_after_fork(self, _after_fork)
+
+            if self._semlock.name is not None:
+                # We only get here if we are on Unix with forking
+                # disabled.  When the object is garbage collected or the
+                # process shuts down we unlink the semaphore name
+                Finalize(self, sem_unlink, (self._semlock.name,),
+                         exitpriority=0)
+                # In case of abnormal termination unlink semaphore name
+                _cleanup_semaphore_if_leaked(self._semlock.name)
 
     def _make_methods(self):
         self.acquire = self._semlock.acquire
@@ -95,7 +123,7 @@ class SemLock(object):
 
     def __setstate__(self, state):
         self._semlock = _billiard.SemLock._rebuild(*state)
-        debug('recreated blocker with handle %r' % state[0])
+        debug('recreated blocker with handle %r', state[0])
         self._make_methods()
 
     @staticmethod
@@ -103,9 +131,6 @@ class SemLock(object):
         return '/%s-%s-%s' % (current_process()._semprefix,
                               os.getpid(), SemLock._counter.next())
 
-#
-# Semaphore
-#
 
 class Semaphore(SemLock):
 
@@ -122,9 +147,6 @@ class Semaphore(SemLock):
             value = 'unknown'
         return '<Semaphore(value=%s)>' % value
 
-#
-# Bounded semaphore
-#
 
 class BoundedSemaphore(Semaphore):
 
@@ -139,11 +161,11 @@ class BoundedSemaphore(Semaphore):
         return '<BoundedSemaphore(value=%s, maxvalue=%s)>' % \
                (value, self._semlock.maxvalue)
 
-#
-# Non-recursive lock
-#
 
 class Lock(SemLock):
+    '''
+    Non-recursive lock.
+    '''
 
     def __init__(self):
         SemLock.__init__(self, SEMAPHORE, 1, 1)
@@ -164,11 +186,11 @@ class Lock(SemLock):
             name = 'unknown'
         return '<Lock(owner=%s)>' % name
 
-#
-# Recursive lock
-#
 
 class RLock(SemLock):
+    '''
+    Recursive lock
+    '''
 
     def __init__(self):
         SemLock.__init__(self, RECURSIVE_MUTEX, 1, 1)
@@ -190,11 +212,11 @@ class RLock(SemLock):
             name, count = 'unknown', 'unknown'
         return '<RLock(%s, %s)>' % (name, count)
 
-#
-# Condition variable
-#
 
 class Condition(object):
+    '''
+    Condition variable
+    '''
 
     def __init__(self, lock=None):
         self._lock = lock or RLock()
@@ -245,7 +267,7 @@ class Condition(object):
 
         try:
             # wait for notification or timeout
-            self._wait_semaphore.acquire(True, timeout)
+            ret = self._wait_semaphore.acquire(True, timeout)
         finally:
             # indicate that this thread has woken
             self._woken_count.release()
@@ -253,6 +275,7 @@ class Condition(object):
             # reacquire lock
             for i in xrange(count):
                 self._lock.acquire()
+            return ret
 
     def notify(self):
         assert self._lock._semlock._is_mine(), 'lock is not owned'
@@ -264,9 +287,9 @@ class Condition(object):
             res = self._sleeping_count.acquire(False)
             assert res
 
-        if self._sleeping_count.acquire(False): # try grabbing a sleeper
-            self._wait_semaphore.release()      # wake up one sleeper
-            self._woken_count.acquire()         # wait for the sleeper to wake
+        if self._sleeping_count.acquire(False):  # try grabbing a sleeper
+            self._wait_semaphore.release()       # wake up one sleeper
+            self._woken_count.acquire()          # wait for sleeper to wake
 
             # rezero _wait_semaphore in case a timeout just happened
             self._wait_semaphore.acquire(False)
@@ -294,9 +317,24 @@ class Condition(object):
             while self._wait_semaphore.acquire(False):
                 pass
 
-#
-# Event
-#
+    def wait_for(self, predicate, timeout=None):
+        result = predicate()
+        if result:
+            return result
+        if timeout is not None:
+            endtime = _time() + timeout
+        else:
+            endtime = None
+            waittime = None
+        while not result:
+            if endtime is not None:
+                waittime = endtime - _time()
+                if waittime <= 0:
+                    break
+            self.wait(waittime)
+            result = predicate()
+        return result
+
 
 class Event(object):
 
@@ -390,11 +428,11 @@ if sys.platform != 'win32':
         except:
             MAXFD = 256
         closerange(0, r)
-        closerange(r+1, MAXFD)
+        closerange(r + 1, MAXFD)
 
         # collect data written to pipe
         data = []
-        while True:
+        while 1:
             try:
                 s = os.read(r, 512)
             except:
